@@ -28,7 +28,7 @@ extern "C" {
 #include <pan/general/e-util.h>
 #include <pan/general/file-util.h>
 #include <pan/general/foreach.h>
-#include <pan/data/scorefile.h>
+#include <pan/usenet-utils/scorefile.h>
 #include <pan/tasks/task-article.h>
 #include <pan/tasks/task-groups.h>
 #include <pan/tasks/task-xover.h>
@@ -96,10 +96,17 @@ namespace
 
   void set_visible (GtkWidget * w, bool visible)
   {
-    if (visible)
-      gtk_widget_show (w);
-    else
+    if (!visible)
       gtk_widget_hide (w);
+    else if (GTK_IS_WINDOW(w))
+      gtk_window_present (GTK_WINDOW(w));
+    else
+      gtk_widget_show (w);
+  }
+
+  void toggle_visible (GtkWidget * w)
+  {
+    set_visible (w, !GTK_WIDGET_VISIBLE(w));
   }
 }
 
@@ -317,6 +324,17 @@ GUI :: GUI (Data& data, Queue& queue, ArticleCache& cache, Prefs& prefs, GroupPr
   _prefs.add_listener (this);
 
   gtk_accel_map_load (get_accel_filename().c_str());
+
+  { // make sure taskbar views have the right tasks in them -- 
+    // when Pan first starts, the active tasks are already running
+    Queue::task_states_t task_states;
+    queue.get_all_task_states(task_states);    
+    foreach(Queue::tasks_t, task_states.tasks, it) {
+      Queue::TaskState s = task_states.get_state(*it);
+      if (s == Queue::RUNNING || s == Queue::DECODING)
+        on_queue_task_active_changed (queue, *(*it), true);
+    }
+  }
 }
 
 namespace
@@ -593,13 +611,14 @@ namespace
 void GUI :: do_show_task_window ()
 {
   static TaskPane * task_pane (0);
-  if (task_pane)
-    gtk_window_present (GTK_WINDOW(task_pane->root()));
-  else {
+
+  if (!task_pane) {
     task_pane = new TaskPane (_queue, _prefs);
-    g_signal_connect (task_pane->root(), "destroy", G_CALLBACK(task_pane_destroyed_cb), &task_pane);
-    gtk_widget_show (task_pane->root());
+    g_signal_connect (task_pane->root(), "destroy",
+                      G_CALLBACK(task_pane_destroyed_cb), &task_pane);
   }
+
+  toggle_visible (task_pane->root());
 }
 
 namespace
@@ -641,13 +660,15 @@ void GUI :: on_log_entry_added (const Log::Entry& e)
 
 void GUI :: do_show_log_window ()
 {
-  static GtkWidget * log_window (0);
-  if (!log_window) {
-    log_window = log_dialog_new (_prefs, get_window (_root));
-    g_signal_connect (log_window, "destroy", G_CALLBACK(gtk_widget_destroyed), &log_window);
-  }
-  gtk_window_present (GTK_WINDOW(log_window));
   set_bin_child (_event_log_button, _info_image);
+
+  static GtkWidget * w (0);
+  if (!w) {
+    w = log_dialog_new (_prefs, get_window (_root));
+    g_signal_connect (w, "destroy", G_CALLBACK(gtk_widget_destroyed), &w);
+  }
+
+  toggle_visible (w);
 }
 void GUI :: do_select_all_articles ()
 {
@@ -1142,13 +1163,13 @@ void GUI :: do_tip_jar ()
 void GUI :: do_about_pan ()
 {
 #if GTK_CHECK_VERSION(2,6,0)
-  const gchar * authors [] = { "Charles Kerr", 0 };
+  const gchar * authors [] = { "Charles Kerr <charles@rebelbase.com>", "Calin Culianu <calin@ajvar.org> - Threaded Decoding", 0 };
   GdkPixbuf * logo = gdk_pixbuf_new_from_inline(-1, icon_pan_about_logo, 0, 0);
   GtkAboutDialog * w (GTK_ABOUT_DIALOG (gtk_about_dialog_new ()));
   gtk_about_dialog_set_name (w, _("Pan"));
   gtk_about_dialog_set_version (w, PACKAGE_VERSION);
   gtk_about_dialog_set_comments (w, VERSION_TITLE);
-  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2006 Charles Kerr"));
+  gtk_about_dialog_set_copyright (w, _("Copyright © 2002-2007 Charles Kerr"));
   gtk_about_dialog_set_website (w, "http://pan.rebelbase.com/");
   gtk_about_dialog_set_logo (w, logo);
   gtk_about_dialog_set_license (w, LICENSE);
@@ -1168,7 +1189,7 @@ void GUI :: do_about_pan ()
   gtk_box_pack_start (box, gtk_image_new_from_pixbuf (logo), false, false, PAD);
   gtk_box_pack_start (box, gtk_label_new("Pan " PACKAGE_VERSION), false, false, PAD);
   gtk_box_pack_start (box, gtk_label_new(VERSION_TITLE), false, false, 0);
-  gtk_box_pack_start (box, gtk_label_new(_("Copyright © 2002-2006 Charles Kerr")), false, false, 0);
+  gtk_box_pack_start (box, gtk_label_new(_("Copyright © 2002-2007 Charles Kerr")), false, false, 0);
   gtk_box_pack_start (box, gtk_label_new("http://pan.rebelbase.com/"), false, false, PAD);
   gtk_widget_show_all (dialog);
   g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
