@@ -17,29 +17,25 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-#include <config.h>
-#include <cassert>
-#include <cctype>
-#include <cmath>
-#include <algorithm>
-#include <functional>
 extern "C" {
+  #include <config.h>
   #include <glib/gi18n.h>
   #include <gtk/gtk.h>
 }
+#include <cctype>
+#include <cmath>
+#include <algorithm>
 #include <pan/general/debug.h>
 #include <pan/general/e-util.h>
 #include <pan/general/foreach.h>
 #include <pan/general/quark.h>
 #include <pan/general/log.h>
-#include <pan/usenet-utils/gnksa.h>
+#include <pan/usenet-utils/filter-info.h>
 #include <pan/data/article.h>
 #include <pan/data/data.h>
-#include <pan/data/filter-info.h>
 #include <pan/icons/pan-pixbufs.h>
 #include "header-pane.h"
-#include "pad.h"
-#include "prefs-ui.h"
+#include "render-bytes.h"
 #include "sexy-icon-entry.h"
 #include "tango-colors.h"
 
@@ -49,24 +45,13 @@ using namespace pan;
 *****
 ****/
 
-
 namespace
 {
   enum
   {
-    COL_DATE_STR,
-    COL_STATE,
-    COL_ACTION,
-    COL_SCORE,
-    COL_LINES,
-    COL_BYTES,
-    COL_DATE,
-    COL_ARTICLE_POINTER,
-    COL_COLLATED_SUBJECT,
-    COL_COLLATED_AUTHOR,
-    COL_SUBJECT,
-    COL_SHORT_AUTHOR,
-    N_COLUMNS
+    COL_DATE_STR, COL_STATE, COL_ACTION, COL_SCORE,
+    COL_LINES, COL_BYTES, COL_DATE, COL_ARTICLE_POINTER,
+    COL_SUBJECT, COL_SHORT_AUTHOR, N_COLUMNS
   };
 }
 
@@ -86,12 +71,6 @@ namespace
 {
   typedef std::set<const Article*> articles_t;
 
-  struct Icon
-  {
-    const guint8 * pixbuf_txt;
-    GdkPixbuf * pixbuf;
-  };
-
   enum
   {
     ICON_READ,
@@ -107,19 +86,21 @@ namespace
     ICON_QTY
    };
 
-   Icon _icons[ICON_QTY] =
-   {
-      { icon_article_read,           0 },
-      { icon_article_unread,         0 },
-      { icon_binary_complete,        0 },
-      { icon_binary_complete_read,   0 },
-      { icon_binary_incomplete,      0 },
-      { icon_binary_incomplete_read, 0 },
-      { icon_disk,                   0 },
-      { icon_bluecheck,              0 },
-      { icon_x,                      0 },
-      { icon_empty,                  0 }
-   };
+  struct Icon {
+    const guint8 * pixbuf_txt;
+    GdkPixbuf * pixbuf;
+  } _icons[ICON_QTY] = {
+    { icon_article_read,           0 },
+    { icon_article_unread,         0 },
+    { icon_binary_complete,        0 },
+    { icon_binary_complete_read,   0 },
+    { icon_binary_incomplete,      0 },
+    { icon_binary_incomplete_read, 0 },
+    { icon_disk,                   0 },
+    { icon_bluecheck,              0 },
+    { icon_x,                      0 },
+    { icon_empty,                  0 }
+  };
 
   const int
   get_article_state (const Data& data, const Article * a)
@@ -140,7 +121,7 @@ namespace
       retval = ICON_READ;
     else
       retval = ICON_UNREAD;
-                                                                                                                                              
+
     return retval;
   }
 
@@ -243,7 +224,6 @@ HeaderPane :: render_score (GtkTreeViewColumn * col,
   }
 
   g_object_set (renderer, "text", buf,
-  //                        "weight", PANGO_WEIGHT_NORMAL,
                           "background", (bg.empty() ? NULL : bg.c_str()),
                           "foreground", (fg.empty() ? NULL : fg.c_str()),
                           NULL);
@@ -258,22 +238,7 @@ HeaderPane :: render_bytes (GtkTreeViewColumn * col,
 {
   unsigned long bytes (0);
   gtk_tree_model_get (model, iter, COL_BYTES, &bytes, -1);
-
-  static const unsigned long KIBI (1024ul);
-  static const unsigned long MEBI (1048576ul);
-  static const unsigned long GIBI (1073741824ul);
-
-  char buf[128];
-  if (bytes < KIBI)
-    g_snprintf (buf, sizeof(buf), "%lu B", bytes);
-  else if (bytes < MEBI)
-    g_snprintf (buf, sizeof(buf), "%.0f KiB", (double)bytes/KIBI);
-  else if (bytes < GIBI)
-    g_snprintf (buf, sizeof(buf), "%.1f MiB", (double)bytes/MEBI);
-  else
-    g_snprintf (buf, sizeof(buf), "%.2f GiB", (double)bytes/GIBI);
-
-  g_object_set (renderer, "text", buf, NULL);
+  g_object_set (renderer, "text", pan::render_bytes(bytes), NULL);
 }
 
 struct HeaderPane::CountUnread: public PanTreeStore::WalkFunctor
@@ -336,11 +301,6 @@ HeaderPane :: render_subject (GtkTreeViewColumn * col,
     NULL);
 }
 
-namespace
-{
-  typedef std::vector<const Article*> article_v;
-}
-
 HeaderPane::Row*
 HeaderPane :: get_row (const Quark& message_id)
 {
@@ -363,6 +323,11 @@ HeaderPane :: create_row (const EvolutionDateMaker & e,
   return row;
 }
 
+namespace
+{
+  typedef std::vector<const Article*> article_v;
+}
+
 void
 HeaderPane ::  add_children_to_model (PanTreeStore               * store,
                                       PanTreeStore::Row          * parent_row,
@@ -483,8 +448,6 @@ HeaderPane :: build_model (const Quark               & group,
     G_TYPE_ULONG,   // bytes
     G_TYPE_ULONG,   //  date
     G_TYPE_POINTER, // article pointer
-    G_TYPE_STRING,  // collated subject (lazy)
-    G_TYPE_STRING,  // collated author (lazy)
     G_TYPE_STRING,  // subject
     G_TYPE_STRING); // short author
 
@@ -527,9 +490,14 @@ HeaderPane :: rebuild ()
 
   _mid_to_row.clear ();
   _tree_store = build_model (_group, _atree, NULL);
+
+  const bool sort_ascending = _prefs.get_flag ("header-pane-sort-ascending", false);
+  int sort_column = _prefs.get_int ("header-pane-sort-column", COL_DATE);
+  if (sort_column<0 || sort_column>=N_COLUMNS) // safeguard against odd settings
+    sort_column = COL_DATE;
   gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE(_tree_store),
-                                        _prefs.get_int ("header-pane-sort-column", COL_DATE),
-                                        (_prefs.get_flag ("header-pane-sort-ascending", false) ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING));
+                                        sort_column,
+                                        sort_ascending ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING);
 
   GtkTreeModel * model (GTK_TREE_MODEL (_tree_store));
   GtkTreeView * view (GTK_TREE_VIEW (_tree_view));
@@ -540,7 +508,7 @@ HeaderPane :: rebuild ()
     gtk_tree_view_expand_all (view);
 
   if (!selectme.empty())
-    select_message_id (*selectme.begin(), true);
+    select_message_id (*selectme.begin());
 }
 
 bool
@@ -653,14 +621,13 @@ HeaderPane :: select_message_id (const Quark& mid, bool do_scroll)
   HeaderPane::Row * row = get_row (mid);
   GtkTreePath * path (_tree_store->get_path(row));
   GtkTreeView * view (GTK_TREE_VIEW(_tree_view));
-  gtk_tree_view_expand_row (view, path, true);
   gtk_tree_view_expand_to_path (view, path);
+  GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
+  gtk_tree_selection_select_path (sel, path);
   if (do_scroll) {
     gtk_tree_view_set_cursor (view, path, NULL, false);
     gtk_tree_view_scroll_to_cell (view, path, NULL, true, 0.5f, 0.0f);
   }
-  GtkTreeSelection * sel (gtk_tree_view_get_selection (view));
-  gtk_tree_selection_select_path (sel, path);
   gtk_tree_path_free (path);
 }
 
@@ -678,6 +645,21 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
     if (!diffs.removed.count ((*it)->message_id))
         new_selection.insert ((*it)->message_id);
 
+  // if the old selection survived,
+  // is it visible on the screen?
+  bool selection_was_visible (true);
+  if (!new_selection.empty()) {
+    GtkTreeView *view (GTK_TREE_VIEW(_tree_view));
+    Row * row (get_row (*new_selection.begin()));
+    GtkTreePath *a(0), *b(0), *p (_tree_store->get_path (row));
+    gtk_tree_view_get_visible_range (view, &a, &b);
+    selection_was_visible = (gtk_tree_path_compare(a,p)<=0 &&
+                             gtk_tree_path_compare(p,b)<=0);
+    gtk_tree_path_free (a);
+    gtk_tree_path_free (b);
+    gtk_tree_path_free (p);
+  }
+
   // if none of the current selection survived,
   // we need to select something to replace the
   // current selection.
@@ -738,11 +720,12 @@ HeaderPane :: on_tree_change (const Data::ArticleTree::Diffs& diffs)
 
   // update our selection if necessary.
   // if the new selection has just been added or reparented,
-  // scroll to it to ensure that it's visible on the screen.
+  // and it was visible on the screen before,
+  // then scroll to ensure it's still visible.
   if (!new_selection.empty()) {
-    const Quark mid (*new_selection.begin());
-    const bool do_scroll = diffs.added.count(mid) || diffs.reparented.count(mid);
-    select_message_id (mid, do_scroll);
+    const bool do_scroll = selection_was_visible
+            && (!diffs.added.empty() || !diffs.reparented.empty() || !diffs.removed.empty());
+    select_message_id (*new_selection.begin(), do_scroll);
   }
 }
 
@@ -1366,8 +1349,8 @@ HeaderPane :: build_tree_columns ()
     else if (name == "subject")
     {
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
-        "ypad", 0,
         "xpad", xpad,
+        "ypad", 0,
         NULL));
       ellipsize_if_supported (G_OBJECT(r));
       col = gtk_tree_view_column_new_with_attributes (_("Subject"), r, NULL);
@@ -1382,8 +1365,8 @@ HeaderPane :: build_tree_columns ()
     else if (name == "score")
     {
       GtkCellRenderer * r = GTK_CELL_RENDERER (g_object_new (GTK_TYPE_CELL_RENDERER_TEXT,
-        "ypad", 0,
         "xpad", xpad,
+        "ypad", 0,
         "xalign", 1.0,
         NULL));
       ellipsize_if_supported (G_OBJECT(r));
@@ -1707,120 +1690,24 @@ namespace
   ***  Navigation Functors
   **/
 
-  struct TreeIteratorNext: public TreeIterFunctor
-  {
+  struct TreeIteratorNext: public TreeIterFunctor {
     virtual ~TreeIteratorNext () {}
-
-    virtual bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const
-    {
-      g_assert (model != 0);
-      g_assert (iter != 0);
-
-      // childen
-      GtkTreeIter tmp;
-      if (gtk_tree_model_iter_nth_child (model, &tmp, iter, 0)) {
-        *iter = tmp;
-        return true;
-      }
-
-      // siblings
-      GtkTreeIter up = *iter;
-      do {
-        tmp = up;
-        if (gtk_tree_model_iter_next (model, &up)) {
-          *iter = up;
-          return true;
-        }
-      } while (gtk_tree_model_iter_parent (model, &up, &tmp));
-
-      // if all else fails, return the first node
-      if (gtk_tree_model_get_iter_first (model, iter))
-        return true;
-
-      return false;
+    virtual bool operator ()(GtkTreeModel * model, GtkTreeIter * iter) const {
+      return model && PAN_TREE_STORE(model)->get_next (iter);
     }
-
     virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
-      const bool ok = gtk_tree_model_get_iter_first (model, setme);
-      return ok;
+      return model && PAN_TREE_STORE(model)->front (setme);
     }
   };
 
-  struct TreeIteratorPrev: public TreeIterFunctor
-  {
-    private:
-
-      // only works for iterators of a tree store.
-      // treestore's iter->user_data is a GNode pointer...
-      bool
-      tree_store_iter_equal (const GtkTreeIter * a, const GtkTreeIter * b) const
-      {
-        if (!a && !b) return true;
-        if (!a || !b) return false;
-        return a->user_data == b->user_data;
-      }
-
-      void
-      tree_model_iter_last_descendant (GtkTreeModel * model,
-                                       GtkTreeIter * setme,
-                                       GtkTreeIter * parent) const
-      {
-        GtkTreeIter march (*parent);
-        for (;;) {
-          const int n_children (gtk_tree_model_iter_n_children (model, &march));
-          if (!n_children) {
-            *setme = march;
-            return;
-          }
-          GtkTreeIter tmp;
-          gtk_tree_model_iter_nth_child (model, &tmp, &march, n_children-1);
-          march = tmp;
-        }
-      }
-
-    public:
-
-      virtual ~TreeIteratorPrev () {}
-
-      bool
-      operator()(GtkTreeModel * model, GtkTreeIter * iter) const
-      {
-        // get parent
-        GtkTreeIter parent;
-        const bool has_parent = gtk_tree_model_iter_parent (model, &parent, iter);
-
-        // get sibling
-        GtkTreeIter sibling;
-        gtk_tree_model_iter_children (model, &sibling, (has_parent ? &parent : 0));
-        if (tree_store_iter_equal (&sibling, iter)) { // iter is the first child
-          *iter = parent;
-          return has_parent;
-        }
-
-        // walk along siblings until we find the one prior to iter.
-        // gtk_tree_model_iter_prev() would have been nice here.
-        for (;;) {
-          GtkTreeIter tmp = sibling;
-          g_assert (gtk_tree_model_iter_next (model, &tmp));
-          if (tree_store_iter_equal (&tmp, iter))
-            break;
-          sibling = tmp;
-        }
-
-        // find the absolutely last child of the older sibling
-        tree_model_iter_last_descendant (model, iter, &sibling);
-        return true;
-      }
-
-      // aka gtk_tree_model_get_iter_last (model, setme);
-      virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
-        const int n (gtk_tree_model_iter_n_children (model, NULL));
-        GtkTreeIter tmp;
-        if (!gtk_tree_model_iter_nth_child (model, &tmp, NULL, n-1))
-          return false;
-        tree_model_iter_last_descendant (model, setme, &tmp);
-        return true;
-      }
+  struct TreeIteratorPrev: public TreeIterFunctor {
+    virtual ~TreeIteratorPrev () {}
+    bool operator()(GtkTreeModel * model, GtkTreeIter * iter) const {
+      return model && PAN_TREE_STORE(model)->get_prev (iter);
+    }
+    virtual bool front (GtkTreeModel* model, GtkTreeIter* setme) const {
+      return model && PAN_TREE_STORE(model)->back (setme);
+    }
   };
 
   /**
