@@ -27,7 +27,7 @@ extern "C" {
 #include <pan/general/debug.h>
 #include <pan/general/foreach.h>
 #include <pan/general/messages.h>
-#include <pan/usenet-utils/utf8-utils.h>
+#include <pan/general/utf8-utils.h>
 #include <pan/data/data.h>
 #include "nntp.h"
 #include "task-xover.h"
@@ -71,7 +71,7 @@ namespace
       snprintf (buf, sizeof(buf), _("Getting all headers for \"%s\""), group.c_str());
     else if (mode == TaskXOver::NEW)
       snprintf (buf, sizeof(buf), _("Getting new headers for \"%s\""), group.c_str());
-    else // SAMPLE 
+    else // SAMPLE
       snprintf (buf, sizeof(buf), _("Sampling headers for \"%s\""), group.c_str());
     return std::string (buf);
   }
@@ -233,7 +233,7 @@ namespace
     if (!view.empty()) {
       errno = 0;
       ul = strtoul (view.str, 0, 10);
-      if (errno) 
+      if (errno)
         ul = 0ul;
     }
 
@@ -369,6 +369,8 @@ TaskXOver :: get_bytes_remaining () const
     minitasks_left += it->second.size();
 
   const double percent_done (_total_minitasks ? (1.0 - minitasks_left/(double)_total_minitasks) : 0.0);
+  if (percent_done < 0.1) // impossible to estimate
+    return 0;
   const unsigned long total_bytes = (unsigned long)(_bytes_so_far / percent_done);
   return total_bytes - _bytes_so_far;
 }
